<!-- Autogenerated by mlir-tblgen; don't manually edit -->
# 'tf_saved_model' Dialect

Dialect used to represent TensorFlow SavedModel in MLIR.
This dialect's main responsibility is to represent the additional
information present in the SavedModel's SavedObjectGraph, which
describes the public interface of this SavedModel.

Module's using this dialect should have the module attribute
`tf_saved_model.semantics`.

This dialect provides an attribute
`tf_saved_model.exported_names = <array of strings>` which indicates which
objects in the module are exported, and under what names. The names
are derived from the object's path in the SavedObjectGraph. For example,
a Python `tf.Module` with an exported function "foo" will result in an MLIR
module where the function has an exported name "foo". If the `tf.Module`
contains a subobject "self.bar" which itself has an exported function
"baz", then the function will have an exported name "bar.baz".
If an object in the object graph is reachable via multiple paths
from the root of the object graph, then this array can have multiple
entries.
TODO(silvasean): This design should be reconsidered after
SavedModel/tf.Module and MLIR linkage semantics are better solidified.
In particular, it would be great if we could assume a single exported name,
and use the standardized MLIR `sym_name` attribute to hold it.

This dialect annotates func ops with two argument attributes
`tf_saved_model.index_path` and `tf_saved_model.bound_input` in order to
model the calling API of a function with SavedModel semantics.
In particular, func's in SavedModel are called with a structured value, and
return a structured value. A structured value consists of arbitrarily nested
dicts/lists with tensors as leaves (composite tensors here mostly behave as
just dicts holding other tensors).
The arity of the Python-level function is modeled as an outer list.
Additionally, any variables or constants used by the function are implicitly
appended to the argument list of the underlying func in a way that is
transparent to the caller.

The function arg/result attribute
`tf_saved_model.index_path = [...some list of strings and integers...]}`
represents the path which one would use to index into a structured
value to reach a given tensor. For example, given the structured value
`[ {"x": leaf0} ]`, the "index path" needed to reach `leaf0` is [0, "x"], as
it would be if you were indexing in Python into such an object
(i.e. `obj[0]["x"]`).

The function arg attribute
`tf_saved_model.bound_input = @some_symbol_name` references a
`tf_saved_model.global_tensor` that is bound to that function argument
and implicitly passed in when the function is invoked from a user-level API.
TODO(silvasean): Consider replacing tf_saved_model.bound_input arg attrs
with "get_global @some_global_tensor" in the function body.

[TOC]

## Operation definition

### `tf_saved_model.asset` (::mlir::tf_saved_model::AssetOp)

Represents an asset in saved model.

Represents an asset in the saved model that points to an external file. It
is a scalar string tensor and it is passed as an argument to the session
initializer functions.

The `sym_name` represents the symbol table name used for internal IR
references.

The `filename` attribute contains the file path to the asset file and it is
relative to saved model directory.

Interfaces: Symbol

#### Attributes:

| Attribute | MLIR Type | Description |
| :-------: | :-------: | ----------- |
| `sym_name` | ::mlir::StringAttr | string attribute
| `filename` | ::mlir::StringAttr | string attribute

### `tf_saved_model.global_tensor` (::mlir::tf_saved_model::GlobalTensorOp)

Represents a global tensor value.

Represents a tensor that is not bound to the lifetime of any particular
function. Such tensors can be marked as mutable via the `is_mutable`
attribute.

These tensors are bound to the arguments of func ops via the
`tf_saved_model.bound_input` argument attr.

The `sym_name` represents the symbol table name used for internal IR
references. The externally visible names, if any, are represented via
a `tf_saved_model.exported_names` attribute.

The `value` attribute contains the tensor's value (or initial value, in the
case it is mutable).

The `type` attribute contains the tensor's type, which for the case of
mutable tensors might be more general than just the fixed static shape of
the `value` attribute. For example, a global tensor might be unranked such
as `tensor<*xf32>`, or a more complex shape such as `tensor<4x?x27xf32>`.
The shape of `value` must be compatible with the shape of `type` in the
sense of `tf.TensorShape` compatibility. And the element types must match.

#### Attributes:

| Attribute | MLIR Type | Description |
| :-------: | :-------: | ----------- |
| `sym_name` | ::mlir::StringAttr | string attribute
| `value` | ::mlir::ElementsAttr | constant vector/tensor attribute
| `type` | ::mlir::TypeAttr | any type attribute
| `is_mutable` | ::mlir::UnitAttr | unit attribute

### `tf_saved_model.session_initializer` (::mlir::tf_saved_model::SessionInitializerOp)

Initializes TensorFlow session state.

The session initializer op marks one or more functions that must be called
by an external agent exactly once to initialize TensorFlow session state,
and this must happen before any other exported functions are called. There
must be no more than one session initializer op in a saved model.

The `initializers` represents the initialization functions. The function
have no output and this function should be only called once.

This is used, for example, to initialize hash tables stored in resources and
accessed by resource name (rather than as resource handles or bound inputs
which is how `global_tensor`s are referenced)

#### Attributes:

| Attribute | MLIR Type | Description |
| :-------: | :-------: | ----------- |
| `initializers` | ::mlir::ArrayAttr | symbol ref array attribute

